@@ -48,8 +48,7 @@ gs_plugin_dummy_init (GsPluginDummy *self)
 	GsPlugin *plugin = GS_PLUGIN (self);
 
 	if (g_getenv ("GS_SELF_TEST_DUMMY_ENABLE") == NULL) {
-		g_debug ("disabling '%s' as not in self test",
-			 gs_plugin_get_name (plugin));
+		g_debug ("disabling itself as not in self test");
 		gs_plugin_set_enabled (plugin, FALSE);
 		return;
 	}
@@ -576,7 +575,7 @@ refine_app (GsPluginDummy        *self,
 			gs_app_set_name (app, GS_APP_QUALITY_NORMAL, "tmp");
 		if (gs_app_get_summary (app) == NULL)
 			gs_app_set_summary (app, GS_APP_QUALITY_NORMAL, "tmp");
-		if (gs_app_get_icons (app) == NULL) {
+		if (!gs_app_has_icons (app)) {
 			g_autoptr(GIcon) ic = g_themed_icon_new ("drive-harddisk");
 			gs_app_add_icon (app, ic);
 		}
