@@ -4,7 +4,7 @@
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  * Copyright (C) 2019 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -66,11 +66,14 @@ struct _GsSummaryTile
 	GsAppTile	 parent_instance;
 
 	GtkWidget	*image;
+	GtkWidget	*image_stack;
 	GtkWidget	*name;
 	GtkWidget	*summary;
 	GtkWidget	*bin;
 	GtkWidget	*stack;
 	gint		 preferred_width;
+
+	GsAppIconsState	 current_app_icons_state;
 };
 
 G_DEFINE_TYPE (GsSummaryTile, gs_summary_tile, GS_TYPE_APP_TILE)
@@ -85,8 +88,8 @@ static void
 gs_summary_tile_refresh (GsAppTile *self)
 {
 	GsSummaryTile *tile = GS_SUMMARY_TILE (self);
+	GsAppIconsState app_icons_state;
 	GsApp *app = gs_app_tile_get_app (self);
-	g_autoptr(GIcon) icon = NULL;
 	gboolean installed;
 	g_autofree gchar *name = NULL;
 	const gchar *summary;
@@ -104,11 +107,29 @@ gs_summary_tile_refresh (GsAppTile *self)
 	gtk_label_set_label (GTK_LABEL (tile->summary), summary);
 	gtk_widget_set_visible (tile->summary, summary && summary[0]);
 
-	icon = gs_app_get_icon_for_size (app,
-					 gtk_image_get_pixel_size (GTK_IMAGE (tile->image)),
-					 gtk_widget_get_scale_factor (tile->image),
-					 "system-component-application");
-	gtk_image_set_from_gicon (GTK_IMAGE (tile->image), icon);
+	app_icons_state = gs_app_get_icons_state (app);
+	if (tile->current_app_icons_state != app_icons_state) {
+		g_autoptr(GIcon) icon = NULL;
+
+		switch (app_icons_state) {
+		case GS_APP_ICONS_STATE_AVAILABLE:
+			icon = gs_app_get_icon_for_size (app,
+							 gtk_image_get_pixel_size (GTK_IMAGE (tile->image)),
+							 gtk_widget_get_scale_factor (tile->image),
+							 "system-component-application");
+			gtk_image_set_from_gicon (GTK_IMAGE (tile->image), icon);
+			gtk_stack_set_visible_child_name (GTK_STACK (tile->image_stack), "image");
+			break;
+		case GS_APP_ICONS_STATE_UNKNOWN:
+		case GS_APP_ICONS_STATE_PENDING_DOWNLOAD:
+		case GS_APP_ICONS_STATE_DOWNLOADING:
+		default:
+			gtk_stack_set_visible_child_name (GTK_STACK (tile->image_stack), "loading");
+			break;
+		}
+
+		tile->current_app_icons_state = app_icons_state;
+	}
 
 	switch (gs_app_get_state (app)) {
 	case GS_APP_STATE_INSTALLED:
@@ -149,10 +170,25 @@ gs_summary_tile_refresh (GsAppTile *self)
 static void
 gs_summary_tile_init (GsSummaryTile *tile)
 {
+	tile->current_app_icons_state = GS_APP_ICONS_STATE_UNKNOWN;
 	tile->preferred_width = -1;
 	gtk_widget_init_template (GTK_WIDGET (tile));
 }
 
+static void
+gs_summary_tile_notify (GObject    *object,
+                        GParamSpec *pspec)
+{
+	GsSummaryTile *self = GS_SUMMARY_TILE (object);
+
+	/* If the app of this tile changes, we have to reload its icon */
+	if (g_strcmp0 (pspec->name, "app") == 0)
+		self->current_app_icons_state = GS_APP_ICONS_STATE_UNKNOWN;
+
+	if (G_OBJECT_CLASS (gs_summary_tile_parent_class)->notify)
+		G_OBJECT_CLASS (gs_summary_tile_parent_class)->notify (object, pspec);
+}
+
 static void
 gs_summary_tile_get_property (GObject *object,
 				 guint prop_id,
@@ -203,6 +239,7 @@ gs_summary_tile_class_init (GsSummaryTileClass *klass)
 
 	object_class->get_property = gs_summary_tile_get_property;
 	object_class->set_property = gs_summary_tile_set_property;
+	object_class->notify = gs_summary_tile_notify;
 
 	tile_class->refresh = gs_summary_tile_refresh;
 
@@ -234,6 +271,8 @@ gs_summary_tile_class_init (GsSummaryTileClass *klass)
 
 	gtk_widget_class_bind_template_child (widget_class, GsSummaryTile,
 					      image);
+	gtk_widget_class_bind_template_child (widget_class, GsSummaryTile,
+					      image_stack);
 	gtk_widget_class_bind_template_child (widget_class, GsSummaryTile,
 					      name);
 	gtk_widget_class_bind_template_child (widget_class, GsSummaryTile,
