@@ -14,6 +14,7 @@
 
 #include "gs-app-list-private.h"
 #include "gs-app-row.h"
+#include "gs-application.h"
 #include "gs-page.h"
 #include "gs-common.h"
 #include "gs-progress-button.h"
@@ -241,8 +242,9 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC(GsUpdatesSectionUpdateHelper, _update_helper_free)
 static void
 _cancel_trigger_failed_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 {
-	GsUpdatesSection *self = GS_UPDATES_SECTION (user_data);
+	g_autoptr(GsUpdatesSection) self = GS_UPDATES_SECTION (g_steal_pointer (&user_data));
 	g_autoptr(GError) error = NULL;
+
 	if (!gs_plugin_loader_job_action_finish (self->plugin_loader, res, &error)) {
 		g_warning ("failed to cancel trigger: %s", error->message);
 		return;
@@ -252,7 +254,7 @@ _cancel_trigger_failed_cb (GObject *source, GAsyncResult *res, gpointer user_dat
 static void
 _reboot_failed_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 {
-	GsUpdatesSection *self = GS_UPDATES_SECTION (user_data);
+	g_autoptr(GsUpdatesSection) self = GS_UPDATES_SECTION (g_steal_pointer (&user_data));
 	g_autoptr(GError) error = NULL;
 	GsApp *app = NULL;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
@@ -266,15 +268,20 @@ _reboot_failed_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 	else if (error != NULL)
 		g_warning ("Calling reboot failed: %s", error->message);
 
-	/* cancel trigger */
-	app = gs_app_list_index (self->list, 0);
-	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_UPDATE_CANCEL,
-					 "app", app,
-					 NULL);
-	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
-					    gs_app_get_cancellable (app),
-					    _cancel_trigger_failed_cb,
-					    self);
+	/* Cancel trigger. The app list might have changed while this async
+	 * function was in flight. If so, ignore it. */
+	if (self->list != NULL && gs_app_list_length (self->list) > 0) {
+		app = gs_app_list_index (self->list, 0);
+		plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_UPDATE_CANCEL,
+						 "app", app,
+						 NULL);
+		gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
+						    gs_app_get_cancellable (app),
+						    _cancel_trigger_failed_cb,
+						    g_object_ref (self));
+	} else {
+		g_debug ("App list changed while waiting for reboot; not cancelling reboot trigger.");
+	}
 }
 
 static gboolean
@@ -389,7 +396,7 @@ _perform_update_cb (GsPluginLoader *plugin_loader, GAsyncResult *res, gpointer u
 
 	/* trigger reboot if any app was not updatable live */
 	if (helper->do_reboot) {
-		gs_utils_invoke_reboot_async (NULL, _reboot_failed_cb, self);
+		gs_utils_invoke_reboot_async (NULL, _reboot_failed_cb, g_object_ref (self));
 
 	/* when we are not doing an offline update, show a notification
 	 * if any app requires a reboot */
@@ -419,16 +426,30 @@ _button_cancel_clicked_cb (GsUpdatesSection *self)
 static void
 _download_finished_cb (GObject *object, GAsyncResult *res, gpointer user_data)
 {
-	g_autoptr(GsUpdatesSection) self = (GsUpdatesSection *) user_data;
+	g_autoptr(GsUpdatesSectionUpdateHelper) helper = user_data;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsAppList) list = NULL;
+	GsUpdatesSection *self = helper->self;
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (object);
 
 	/* get result */
-	list = gs_plugin_loader_job_process_finish (GS_PLUGIN_LOADER (object), res, &error);
+	list = gs_plugin_loader_job_process_finish (plugin_loader, res, &error);
 	if (list == NULL) {
 		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) &&
-		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-			g_warning ("failed to download updates: %s", error->message);
+		    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			gs_plugin_loader_claim_job_error (plugin_loader,
+							  NULL,
+							  helper->job,
+							  error);
+		}
+	} else if (!gs_page_is_active_and_focused (self->page)) {
+		g_autoptr(GNotification) notif = NULL;
+
+		notif = g_notification_new (_("Software Updates Downloaded"));
+		g_notification_set_body (notif, _("Software updates have been downloaded and are ready to be installed."));
+		g_notification_set_default_action_and_target (notif, "app.set-mode", "s", "updates");
+		/* last the notification for an hour */
+		gs_application_send_notification (GS_APPLICATION (g_application_get_default ()), "updates-downloaded", notif, 60);
 	}
 
 	g_clear_object (&self->cancellable);
@@ -440,14 +461,21 @@ _button_download_clicked_cb (GsUpdatesSection *self)
 {
 	g_autoptr(GCancellable) cancellable = g_cancellable_new ();
 	g_autoptr(GsPluginJob) plugin_job = NULL;
+	GsUpdatesSectionUpdateHelper *helper;
+
+	g_application_withdraw_notification (g_application_get_default (), "updates-downloaded");
 
 	g_set_object (&self->cancellable, cancellable);
 	plugin_job = gs_plugin_job_update_apps_new (self->list,
 						    GS_PLUGIN_UPDATE_APPS_FLAGS_NO_APPLY | GS_PLUGIN_UPDATE_APPS_FLAGS_INTERACTIVE);
+	gs_plugin_job_set_propagate_error (plugin_job, TRUE);
+	helper = g_new0 (GsUpdatesSectionUpdateHelper, 1);
+	helper->self = g_object_ref (self);
+	helper->job = g_object_ref (plugin_job);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
 					    self->cancellable,
-					    (GAsyncReadyCallback) _download_finished_cb,
-					    g_object_ref (self));
+					    _download_finished_cb,
+					    helper);
 	_update_buttons (self);
 }
 
@@ -460,12 +488,17 @@ _button_update_all_clicked_cb (GsUpdatesSection *self)
 
 	helper->self = g_object_ref (self);
 
+	if (self->kind == GS_UPDATES_SECTION_KIND_OFFLINE_FIRMWARE ||
+	    self->kind == GS_UPDATES_SECTION_KIND_OFFLINE) {
+		helper->do_reboot = TRUE;
+	}
+
 	/* look at each app in turn */
-	for (guint i = 0; i < gs_app_list_length (self->list); i++) {
+	for (guint i = 0; (!helper->do_reboot || !helper->do_reboot_notification) && i < gs_app_list_length (self->list); i++) {
 		GsApp *app = gs_app_list_index (self->list, i);
-		if (gs_app_get_state (app) == GS_APP_STATE_UPDATABLE)
+		if (!helper->do_reboot && gs_app_get_state (app) == GS_APP_STATE_UPDATABLE)
 			helper->do_reboot = TRUE;
-		if (gs_app_has_quirk (app, GS_APP_QUIRK_NEEDS_REBOOT))
+		if (!helper->do_reboot_notification && gs_app_has_quirk (app, GS_APP_QUIRK_NEEDS_REBOOT))
 			helper->do_reboot_notification = TRUE;
 	}
 
